function [X,lambda]=qpopt(H,f,A,B,vlb,vub,X0,neqcstr)
%QP Quadratic programming (using GAMS). 
%   X=QP(H,f,A,b) solves the quadratic programming problem:
%
%            min 0.5*x'Hx + f'x   subject to:  Ax <= b 
%             x    
%
%       X=QP(H,f,A,b,VLB,VUB) defines a set of lower and upper
%       bounds on the design variables, X, so that the solution  
%       is always in the range VLB <= X <= VUB.
%
%       X=QP(H,f,A,b,VLB,VUB,X0) sets the initial starting point to X0.
%
%       X=QP(H,f,A,b,VLB,VUB,X0,N) indicates that the first N constraints 
%       defined by A and b are equality constraints.
%
%       [x,LAMBDA]=QP(H,f,A,b) returns the set of Lagrangian multipliers,
%       LAMBDA, at the solution.
%

% Handle missing arguments
   if nargin < 8, neqcstr = 0; 
      if nargin < 7, X0 = []; 
         if nargin < 6, vub = []; 
           if nargin < 5, vlb = [];
end, end, end, end
[ncstr,nvars]=size(A);
nvars = max([length(f),length(H),nvars]); % In case A is empty

if isempty(neqcstr), neqcstr = 0; end
if isempty(X0), X0=zeros(nvars,1); end

if isempty(vlb), vlb=-inf*ones(nvars,1); end
if isempty(vub), vub=inf*ones(nvars,1); end

if isempty(A), A=zeros(0,nvars); end
if isempty(B), B=zeros(0,1); end

% Expect vectors
f=f(:);
B=B(:);
X0 = X0(:);
vlb = vlb(:);
vub = vub(:);

if  norm(H,'inf')==0 | isempty(H)
  H=[]; 
else
  % Make sure it is symmetric
  if norm(H-H','inf') > eps
    H = (H+H')*0.5;
  end
end

% fool interface to make A a matrix, b a vector
if ncstr == 1 
  A(ncstr+1,1) = 0;
  B(ncstr+1) = 0;
end
neqcstr = int2str(neqcstr);
ncstr = int2str(ncstr);
nvars = int2str(nvars);
gams_output = 'std';
[X,lambda] = gams('qpopt','H','f','A','B','vlb','vub','X0','neqcstr','ncstr','nvars');


